"""
NavConfig.

Main object for Configuration of several Navigator-related Tools.
"""
import sys
import os
from pathlib import Path
from .version import (
    __title__, __description__, __version__, __author__, __author_email__
)
from .config import navigatorConfig # noqa

def is_virtualenv():
    return (
        hasattr(sys, 'real_prefix') or (
            hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix)
    )


# PROJECT PATH IS DEFINED?
SITE_ROOT = os.getenv('SITE_ROOT', None)

if not SITE_ROOT:
    # get Project PATH
    if is_virtualenv():
        SITE_ROOT = Path(sys.prefix).resolve().parent
    else:
        SITE_ROOT = Path(os.path.abspath(
            os.path.dirname(__file__))).resolve().parent.parent
    if not SITE_ROOT:
        SITE_ROOT = Path(sys.prefix).resolve().parent
else:
    SITE_ROOT = Path(SITE_ROOT).resolve()


BASE_DIR = os.getenv('BASE_DIR', None)
if not BASE_DIR:
    BASE_DIR = SITE_ROOT
else:
    BASE_DIR = Path(BASE_DIR).resolve()

SETTINGS_DIR = BASE_DIR.joinpath('settings')

# configuration of the environment type:
ENV_TYPE = os.getenv('ENV_TYPE', 'file')

"""
Loading main Configuration Object.
"""
config = navigatorConfig(SITE_ROOT, env_type = ENV_TYPE)
ENV = config.ENV

# DEBUG VERSION
DEBUG = config.debug

# SECURITY WARNING: keep the secret keys used in production secret!
PRODUCTION = config.getboolean('PRODUCTION', fallback=bool(not DEBUG))

# Add Path Navigator to Sys path
sys.path.append(str(BASE_DIR))
sys.path.append(str(SETTINGS_DIR))



import codecs
import toml
from pathlib import Path
from .base import BaseLoader
from typing import (
    Union
)

class TomlLoader(BaseLoader):
    """TomlLoader.
    
    Used to read configuration settings from TOML files.

    Args:
        BaseLoader (_type_): _description_
    """

    def load_enviroment(self, file: Union[str, Path]):
        with codecs.open(file, 'r') as f:
            content = toml.loads(f.read())
        self.load_from_string(content)

    def save_enviroment(self):
        pass


import hashlib
import aiofiles
from cryptography.fernet import Fernet
from pathlib import PurePath, PosixPath
from io import StringIO

    
class FileCypher(object):
    def __init__(self, directory: PurePath):
        self.path = directory

    async def create_key(self):
        #generate the key
        key = Fernet.generate_key()
        file = self.path.joinpath('unlock.key')
        #string the key into a file
        async with aiofiles.open(file, 'wb') as unlock:
            await unlock.write(key)
        return file
    
    async def open_file(self, path: PurePath):
        content = None
        if not path.exists():
            raise FileNotFoundError(
                f'File {path} does not exist'
            )
        try:
            async with aiofiles.open(path) as f:
                content = await f.read()
        except IOError:
            raise Exception(
                f'NavConfig: Error loading Environment File {path}'
            )
        return content
    
    async def save_file(self, path: PurePath, content):
        async with aiofiles.open(path, 'wb') as file:
            await file.write(content)
    
    async def get_key(self):
        fkey = self.path.joinpath('unlock.key')
        key = None
        async with aiofiles.open(fkey) as f:
            key = await f.read()
        if not key:
            raise Exception(
                f'Missing the Unlock Key: {fkey!s}'
            )
        #use the generated key
        f = Fernet(key)
        return f
    
    async def encrypt(self, name: str = '.env'):
        #use the generated key
        f = await self.get_key()
        file = self.path.joinpath(name)
        # original content
        original = await self.open_file(file)
        #encrypt the file
        encrypted = f.encrypt(original.encode())
        # at now, save it into the same directory
        file = self.path.joinpath('env.crypt')
        await self.save_file(file, encrypted)
        return file
    
    async def decrypt(self, name: str = 'env.crypt'):
        #use the generated key
        f = await self.get_key()
        #open the original file to encrypt
        file = self.path.joinpath(name)
        content = await self.open_file(file)
        #decrypt the file
        decrypted = f.decrypt(content.encode())
        s = StringIO()
        s.write(decrypted.decode())
        s.seek(0)
        # returned a StringIO
        return s


"""NavConfig Meta information."""

__title__ = 'navconfig'
__description__ = ('Configuration tool for all Navigator Services '
                   'Tool for accessing Config info from different sources.')
__version__ = '0.9.2'
__author__ = 'Jesus Lara'
__author_email__ = 'jesuslarag@gmail.com'
__license__ = 'BSD'



import os
import sys
import logging
from pathlib import Path
from navconfig import (
    BASE_DIR
)
os.chdir(str(BASE_DIR))

"""
Global-Settings.
"""
try:
    from settings.settings import *
except (ImportError, ModuleNotFoundError) as err:
    from settings import settings
    logging.error(f'Cannot loaded a settings Module {err}, module: {settings}')
    print(
        'Settings.py module is missing.'
        'Hint: Its recommended to use a settings/settings.py module to customize '
        ' Navigator Configuration.'
    )

"""
User Local Settings
"""
try:
    from settings.local_settings import *
except (ImportError, ModuleNotFoundError) as err:
    pass



import codecs
import yaml
from pathlib import Path
from .base import BaseLoader
from typing import (
    Union
)

class YamlLoader(BaseLoader):
    """YamlLoader.
    
    Used to read configuration settings from YAML files.

    Args:
        BaseLoader (_type_): _description_
    """

    def load_enviroment(self, file: Union[str, Path]):
        with codecs.open(file, 'r') as f:
            content = yaml.safe_load(f)
        self.load_from_string(content)

    def save_enviroment(self):
        pass


#!/usr/bin/env python
"""NavConfig.

    Configuration Service for Navigator and DataIntegrator.
See:
https://github.com/phenobarbital/NavConfig
"""
from os import path
from setuptools import setup, find_packages


def get_path(filename):
    return path.join(path.dirname(path.abspath(__file__)), filename)


def readme():
    with open(get_path('README.md')) as readme:
        return readme.read()


with open(get_path('navconfig/version.py')) as meta:
    exec(meta.read())

setup(
    name=__title__,
    version=__version__,
    python_requires=">=3.8.0",
    url='https://github.com/phenobarbital/NavConfig',
    description=__description__,
    long_description=readme(),
    long_description_content_type='text/markdown',
    license=__license__,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Environment :: Web Environment',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Framework :: AsyncIO'
    ],
    author='Jesus Lara',
    author_email='jesuslara@phenobarbital.info',
    packages=find_packages(),
    setup_requires=[
        'wheel==0.37.1'
    ],
    install_requires=[
        'asyncio==3.4.3',
        'uvloop==0.16.0',
        'python-dotenv==0.20.0',
        'configparser==5.2.0',
        'PyYAML>=6.0',
        'PyDrive==1.3.1',
        'pylibmc==1.6.1',
        'aiomcache==0.7.0',
        'objectpath==0.6.1',
        'iso8601==1.0.2',
        'pycparser==2.21',
        'redis==4.3.4',
        'aioredis==2.0.1',
        'python-rapidjson==1.8',
        'python-logstash-async==2.5.0',
        'aiologstash==2.0.0',
        'pycryptodomex==3.15.0',
        "cryptography==37.0.4",
        'aiofiles==0.8.0'
    ],
    project_urls={  # Optional
        'Source': 'https://github.com/phenobarbital/NavConfig',
        'Funding': 'https://paypal.me/phenobarbital',
        'Say Thanks!': 'https://saythanks.io/to/phenobarbital',
    },
)
